#include <assert.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ansi.h"
#include "chain.h"
#include "concatenate.h"
#include "file_exists.h"
#include "hardware.h"
#include "read_line.h"



TBlockDevice *create_eide_device() {
  return calloc(1, sizeof(TBlockDevice));
}



int query_eide_devices(TComputer *pc) {
  DIR *lpdir = NULL;
  struct dirent *lpdirent = NULL;
  TChainItem *pcurrent = NULL;
  FILE *fd = NULL;
  char *tempstr = NULL;
  tempstr  = calloc( 256, sizeof(char) );

  assert(pc!=NULL);
  
  // Clean Up previous Eide Chain
  if (pc->eide!=NULL) {
    chain_delete(pc->eide);
  }

  // Controlla presenza di /proc/ide
  if ( 0 != file_exists(DIR_PROC_IDE) ) {
    printERROR("Directory %s inesistente !\n", DIR_PROC_IDE );
    normal_color();
    return 1;
  }

  // Apre directory /proc/ide
  lpdir = opendir(DIR_PROC_IDE);
  if (lpdir==NULL) {    
    printERROR("Impossibile aprire directory %s !\n", DIR_PROC_IDE );
    normal_color();
    return 1;
  }

  // Cerca nella directory ...
  lpdirent = readdir(lpdir);
  while (lpdirent!=NULL) {
    // ... i files con maschera '^hd*'
    if ( 0 == strncmp(lpdirent->d_name, "hd", 2 ) ) {

      // Abbiamo trovato 1 device, puoi creare la catena adesso
      if(pc->eide==NULL) pc->eide = chain_create();

      // Alloca memoria per nuovo device
      //printf("trovato= %s \n", lpdirent->d_name );
      pcurrent = chain_append( pc->eide );
      pcurrent->data = create_eide_device();
      // Assegna nome device
      pcurrent->name = concatenate( "/dev/", lpdirent->d_name );
      TBLOCKDEVICE(pcurrent->data)->device = strdup(pcurrent->name);

      // Lettura Modello periferica
      sprintf(tempstr, "%s/%s/model", DIR_PROC_IDE, lpdirent->d_name );
      if ( 0 != file_exists(tempstr) ) {
        printERROR("Impossibile aprire file %s\n", tempstr);
	normal_color();
        return 1;
      }
      fd = fopen(tempstr,"r");
      if (fd!=NULL) {
        read_line(fd, tempstr);
        fclose(fd);
        TBLOCKDEVICE(pcurrent->data)->model = strdup(tempstr);
      } else {
        TBLOCKDEVICE(pcurrent->data)->model = MODEL_UNKNOWN;
      }
      //printf("Model= %s\n", current->model );

      // Lettura Media periferica
      sprintf(tempstr, "%s/%s/media", DIR_PROC_IDE, lpdirent->d_name );
      if ( 0 == file_exists(tempstr) ) {
        fd = fopen(tempstr,"r");
        if (fd==NULL) {
          printERROR("Impossibile aprire file %s\n", tempstr);
	  normal_color();
          return 1;
        }
        read_line(fd, tempstr);
        fclose(fd);
        TBLOCKDEVICE(pcurrent->data)->media = strdup(tempstr);
      } else {
	TBLOCKDEVICE(pcurrent->data)->media = MEDIA_UNKNOWN;
      }
      //printf("Media= %s\n", current->media );
    }
    
    // Next file
    lpdirent = readdir(lpdir);
  }
  closedir(lpdir);
  return 0;
}

