#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "ansi.h"
#include "chain.h"
#include "hardware.h"
#include "read_line.h"
#include "runcommand.h"


TPartition *create_partition() {
  return calloc( 1, sizeof(TPartition));
}


// Look up Partitions from a device using cfdisk
TChain *probe_partitions_harddisk(char *devicename) {
  TChain *lpchain = NULL;
  TChainItem *pcurrent = NULL;
  char *lpaopt[7];
  FILE *fd = NULL;
  int   lnum = 0;
  char *tempstr = NULL;
  char *tempdata = NULL;
  tempstr  = calloc( 256, sizeof(char) );
  tempdata = calloc( 256, sizeof(char) );

  // Unlink tempfile if it exists
  unlink(TEMPFILE_PARTITIONS);
  
  // Prepare structure to run cfdisk
  lpaopt[0] = "cfdisk";
  lpaopt[1] = "-P";	    
  lpaopt[2] = "s";
  lpaopt[3] = devicename;
  lpaopt[4] = 0;

  // Open file for writing
  fd = fopen(TEMPFILE_PARTITIONS, "w");
  if (fd==NULL) {
    printERROR("Impossibile aprire %s in scrittura !\n", TEMPFILE_PARTITIONS);
    normal_color();
    return NULL;
  }
  
  // Run cfdisk, output goes to the tempfile, errors to null
  if ( 0 != runcommand( lpaopt, 0, fileno(fd), -1 )){
    fclose(fd);
    printERROR("Impossibile eseguire cfdisk %s !\n", devicename );
    normal_color();
    return NULL;
  }

  // Reopen output file for writing
  fd = freopen(TEMPFILE_PARTITIONS,"r",fd);
  if (fd==NULL) {
    printERROR("Impossibile aprire file %s in lettura\n", TEMPFILE_PARTITIONS );
    normal_color();
    return NULL;
  }

  // Read output from the file
  while ( ! feof(fd) ) {
    read_line(fd,tempstr);
    // Get only lines with a partition number
    if ( (tempstr[1]<'0') || (tempstr[1]>'9') ) continue;
    //printf(">%s\n", tempstr);

    // --- Decode Partition Name ---
    strncpy(tempdata, tempstr, 2);
    tempdata[2]=0;
    lnum = atol(tempdata);
    if ((lnum<=0)||(lnum>20)) {
      printWARNING("Numero di partizione errato (%i) mentre decifravo la riga %s\n", lnum, tempstr);
      normal_color();
      continue;	
    }

    // Add partition to the chain
    if (lpchain==NULL) lpchain=chain_create();
    pcurrent = chain_append(lpchain);
    pcurrent->name = calloc( strlen(devicename) + 3, sizeof(char) );
    sprintf( pcurrent->name, "%s%i", devicename, lnum );
    //printf("New partition = %s \n", current->name );

    // Add data
    pcurrent->data = create_partition();
    TPARTITION(pcurrent->data)->device = strdup(pcurrent->name);

    // Calcolo dimensione
    strncpy(tempdata, tempstr+37, 8);
    tempdata[8]=0;
    TPARTITION(pcurrent->data)->size = atol(tempdata);
    //printf("Nuova dimensione #%s# = %i\n", tempdata, lnum);
    
    // Calcolo FS Type
    if ( NULL != strstr(tempstr+47,"Linux Swap")) 
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_SWAP;
    else
    if ( NULL != strstr(tempstr+47,"Linux swap")) 
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_SWAP;
    else
    if ( NULL != strstr(tempstr+47,"Linux extended")) 
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_NONE;
    else
    if ( NULL != strstr(tempstr+47,"Linux ")) 
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_EXT2;
    else
    if ( NULL != strstr(tempstr+47,"Win95 Extended")) 
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_NONE;
    else
    if ( NULL != strstr(tempstr+47,"DOS")) 
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_VFAT;
    else
    if ( NULL != strstr(tempstr+47,"Win95 FAT")) 
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_VFAT;
    else
    if ( NULL != strstr(tempstr+47,"FAT")) 
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_VFAT;
    else
      TPARTITION(pcurrent->data)->filesystem=FILESYSTEM_NONE;
    //printf("fstype = %s \n", TPARTITION(pcurrent->data)->filesystem );
  }

  // Close file and exit
  fclose(fd);
  return lpchain;
}



int query_partitions_device(TBlockDevice *blockdevice) {
  TChainItem *pcurrent = NULL;
  assert(blockdevice!=NULL);
  // Remove previous chain
  if (blockdevice->partitions != NULL) {
    chain_delete(blockdevice->partitions);
    blockdevice->partitions = NULL;
  }
  // Check if is a disk
  if ( 0 == strcmp(blockdevice->media,MEDIA_DISK_EIDE)) {
    // Probe Partitions thru' cfdisk
    blockdevice->partitions = probe_partitions_harddisk(blockdevice->device);
    return 0;
  } else
  if ( 0 == strcmp(blockdevice->media,MEDIA_CDROM_EIDE)) {
    // Cdrom have only 1 partition of type ISO
    blockdevice->partitions = chain_create();
    pcurrent = chain_append(blockdevice->partitions);
    pcurrent->name = strdup(blockdevice->device);
    pcurrent->data = create_partition();
    TPARTITION(pcurrent->data)->device     = strdup(blockdevice->device);
    TPARTITION(pcurrent->data)->filesystem = FILESYSTEM_ISO9660;
    TPARTITION(pcurrent->data)->size       = 0;
    return 0;
  }
  // Other device does not have a fixed type file system
  blockdevice->partitions = NULL;
  return 0;
}


int query_partitions(TComputer *computer) {
  TChainItem *item, *pitem, *ppitem;
  TBlockDevice *pdata = NULL;
  assert(computer!=NULL);
  
  // Delete old collection
  if (computer->partitions!=NULL) {
    chain_delete(computer->partitions);
    computer->partitions = NULL;
  }
  
  // Attach partitions from eide devices
  if (computer->eide != NULL ){
    for ( item=computer->eide->head; item!=NULL; item=item->next ) {
      pdata=item->data;
      query_partitions_device(pdata);
      if (pdata->partitions != NULL) {
        for ( pitem=pdata->partitions->head; pitem!=NULL; pitem=pitem->next ) {
	  if (computer->partitions==NULL) computer->partitions=chain_create();
	  ppitem = chain_append(computer->partitions);
	  ppitem->name = strdup(pitem->name);
	  ppitem->data = pitem->data;
	}      
      }
    }
  }

  // Attach partitions from scsi devices
  if (computer->scsi != NULL ){
    for ( item=computer->scsi->head; item!=NULL; item=item->next ) {
      pdata=item->data;
      query_partitions_device(pdata);
      if (pdata->partitions != NULL) {
        for ( pitem=pdata->partitions->head; pitem!=NULL; pitem=pitem->next ) {
	  if (computer->partitions==NULL) computer->partitions=chain_create();
	  ppitem = chain_append(computer->partitions);
	  ppitem->name = strdup(pitem->name);
	  ppitem->data = pitem->data;
	}      
      }
    }
  }

  // done
  return 0;
}
